import time

try:
    from neopixel_plus.helper import Color
except ImportError:
    from helper import Color


class BeatsUpAndDown:
    def __init__(self,
                 led_strip,
                 loop_limit=None,
                 duration_ms=200,
                 pause_ms=300,
                 start='start',
                 rgb_colors=None,
                 brightness=1,
                 brightness_fixed=False,
                 max_height=1,
                 num_random_colors=5,
                 ):
        self.led_strip = led_strip
        self.loop_limit = loop_limit
        self.loops = 0
        self.duration_ms = duration_ms
        self.pause_ms = pause_ms
        self.start = start
        self.max_height = max_height

        self.write_wait_time = (
            self.duration_ms/2/self.led_strip.addressable_strip_length)/1000

        self.colors = Color(
            rgb_colors=rgb_colors,
            brightness=brightness,
            brightness_fixed=brightness_fixed,
            num_random_colors=num_random_colors
        )

        if self.start == 'start' or self.start == 'end':
            self.led_strip.addressable_strip_length = round(
                self.led_strip.strip_length*self.max_height)
            self.selected_leds = range(self.led_strip.addressable_strip_length)
            self.selected_leds_counter_up = self.led_strip.addressable_strip_length
            self.selected_leds_counter_down = round(self.led_strip.addressable_strip_length -
                                                    self.selected_leds_counter_up)
        else:
            self.led_strip.addressable_strip_length = self.led_strip.strip_length
            self.selected_leds = range(
                round(self.led_strip.addressable_strip_length/2))
            self.selected_leds_counter_up = round(self.max_height *
                                                  round(self.led_strip.addressable_strip_length/2))
            self.selected_leds_counter_down = round(round(
                self.led_strip.addressable_strip_length/2)-self.selected_leds_counter_up)

    def color_leds(self):
        if self.led_strip.debug:
            print('BeatsUpAndDown().color_leds()')

        # color LEDs
        for i in self.selected_leds[:self.selected_leds_counter_up]:
            # if brightness_fixed==False: set brightness depending on what led is glowing up
            if self.colors.brightness_fixed == False:
                # led 1: 30% of self.brightness_max
                # led 2: 30% of max + (i * 70%/self.led_strip.strip_length)
                # last LED: 100% * self.brightness_max
                if self.start == 'start + end' or self.start == 'center':
                    self.colors.brightness = round((0.3*self.colors.brightness_max) +
                                                   ((i+1)*(0.7/((self.led_strip.addressable_strip_length*self.max_height)/2))), 2)
                else:
                    self.colors.brightness = round((0.3*self.colors.brightness_max) +
                                                   ((i+1)*(0.7/self.led_strip.addressable_strip_length)), 2)
                self.colors.correct()

            # "start" option "start & end" and "center"
            if self.start == 'end':
                i = [-(i+1)]
            elif self.start == 'start':
                i = [i]
            elif self.start == 'start + end':
                i = [i, -(i+1)]
            elif self.start == 'center':
                i = [i+round(self.led_strip.addressable_strip_length/2), -
                     (i+1)+round(self.led_strip.addressable_strip_length/2)]

            for led in i:
                led = self.led_strip.get_led(led, self.start)
                self.led_strip.leds[led] = self.colors.selected

            self.led_strip.write(s_after_wait=self.write_wait_time)

    def make_leds_black(self):
        if self.led_strip.debug:
            print('BeatsUpAndDown().make_leds_black()')

        # then make them black
        for i in self.selected_leds[self.selected_leds_counter_down:]:
            if self.start == 'start':
                i = [-(i+1)]
            elif self.start == 'end':
                i = [i]
            elif self.start == 'start + end':
                i = [-(i+1)+round(self.led_strip.addressable_strip_length/2),
                     i+round(self.led_strip.addressable_strip_length/2)]
            elif self.start == 'center':
                i = [i, -(i+1)]

            for led in i:
                led = self.led_strip.get_led(led, self.start)
                self.led_strip.leds[led] = self.colors.black

            self.led_strip.write(s_after_wait=self.write_wait_time)

    def glow(self):
        if self.led_strip.debug:
            print('BeatsUpAndDown().glow()')

        print('Beats up and down:')
        try:
            # make sure leds are off
            self.led_strip.off()

            while True:
                # update color if brightness different
                if self.colors.brightness != 1 and self.colors.brightness_fixed:
                    self.colors.correct()

                self.color_leds()
                self.make_leds_black()

                # change to next color
                self.colors.next()

                if self.pause_ms:
                    time.sleep(self.pause_ms/1000)

                self.loops += 1
                if self.loop_limit and self.loop_limit == self.loops:
                    print()
                    break
        except KeyboardInterrupt:
            import sys
            print()
            sys.exit(0)



try:
    from neopixel_plus.animations.beats_up_and_down import BeatsUpAndDown
    from neopixel_plus.animations.light_up import LightUp
    from neopixel_plus.animations.moving_dot import MovingDot
    from neopixel_plus.animations.rainbow import RainbowAnimation
    from neopixel_plus.animations.transition import Transition
except ImportError:
    from animations.beats_up_and_down import BeatsUpAndDown
    from animations.light_up import LightUp
    from animations.moving_dot import MovingDot
    from animations.rainbow import RainbowAnimation
    from animations.transition import Transition



import time

try:
    from neopixel_plus.helper import Color
except ImportError:
    from helper import Color


class LightUp:
    def __init__(self,
                 led_strip,
                 rgb_colors=None,
                 brightness=1,
                 loop_limit=None,
                 duration_ms=200,
                 pause_ms=200,
                 num_random_colors=5,
                 sections='all'):
        self.led_strip = led_strip
        self.loop_limit = loop_limit
        self.loops = 0
        self.duration_ms = duration_ms
        self.pause_ms = pause_ms
        self.sections = sections
        self.colors = Color(
            rgb_colors=rgb_colors,
            brightness=brightness,
            num_random_colors=num_random_colors
        )

        self.write_wait_time = (
            self.duration_ms/((round(self.colors.brightness_max, 1)/0.1)*2)/1000)

    def glow(self):
        print('Light up:')
        try:
            # make sure leds are off
            self.led_strip.off()

            while True:
                # go over levels of brightness from 0 to 1 and
                self.colors.brightness = 0
                self.colors.correct()

                self.selected_leds = self.led_strip.get_led_selectors(
                    self.sections)

                # light up
                while self.colors.brightness != round(self.colors.brightness_max, 1):
                    for i in self.selected_leds:
                        self.led_strip.leds[i] = self.colors.selected
                    self.led_strip.write(s_after_wait=self.write_wait_time)

                    self.colors.brightness = round(
                        self.colors.brightness+0.1, 1)
                    self.colors.correct()

                # light down
                while self.colors.brightness != 0.0:
                    self.colors.brightness = round(
                        self.colors.brightness-0.1, 1)
                    self.colors.correct()
                    for i in self.selected_leds:
                        self.led_strip.leds[i] = self.colors.selected
                    self.led_strip.write(s_after_wait=self.write_wait_time)

                # change to next color
                self.colors.next()

                if self.pause_ms:
                    time.sleep(self.pause_ms/1000)

                self.loops += 1
                if self.loop_limit and self.loop_limit == self.loops:
                    print()
                    break

        except KeyboardInterrupt:
            import sys
            print()
            sys.exit(0)



import random


class Color:
    def __init__(self, rgb_colors, brightness=1, brightness_fixed=False, num_random_colors=5):
        self.rgb_colors = rgb_colors if rgb_colors else self.random_set(
            num_random_colors)
        self.selected_num = 0
        self.selected_max = len(self.rgb_colors)-1
        self.base_color = self.rgb_colors[self.selected_num]
        self.brightness = brightness
        self.brightness_max = brightness
        self.brightness_fixed = brightness_fixed

        self.correct()

    @property
    def black(self):
        return [0, 0, 0]

    @property
    def white(self):
        return [255, 255, 255]

    def random(self):
        return [random.randint(1, 255), random.randint(1, 255), random.randint(1, 255)]

    def random_set(self, how_many):
        array = [0]*how_many
        return [self.random() for x in array]

    def correct(self):
        r = int(self.base_color[0] * self.brightness)
        g = int(self.base_color[1] * self.brightness)
        b = int(self.base_color[2] * self.brightness)

        self.selected = [r if r < 255 else 255, g if g <
                         255 else 255, b if b < 255 else 255]

    def next(self):
        # select next color from self.rgb_colors, with correct brightness
        if self.selected_num == self.selected_max:
            self.selected_num = 0
        else:
            self.selected_num += 1
        self.base_color = self.rgb_colors[self.selected_num]
        self.correct()



import getopt
import math
import random
import sys
import time
from random import randint

try:
    from neopixel_plus.animations import *
except ImportError:
    from animations import *


class NeoPixel:
    def __init__(self,
                 pin_num=None,
                 n=30,
                 start_led=0,
                 test=False,
                 overwrite_line=True,
                 debug=False,
                 target='micropython'  # or 'adafruit'
                 ):
        self.debug = debug
        self.target = target
        self.strip_length = n
        self.addressable_strip_length = n
        self.start_led = start_led
        self.test = test
        self.pin_num = pin_num if pin_num else 10 if self.target == 'micropython' else 18 if self.target == 'adafruit' else None
        self.overwrite_line = overwrite_line
        self.sections = self.get_sections()

        if self.test:
            self.leds = [[0, 0, 0] for x in range(self.strip_length)]
        else:
            from neopixel import NeoPixel as NeoPixelOriginal
            if self.target == 'micropython':
                self.leds = NeoPixelOriginal(
                    self.get_pin(),
                    self.strip_length)
            elif self.target == 'adafruit':
                self.leds = NeoPixelOriginal(
                    self.get_pin(),
                    self.strip_length,
                    auto_write=False)

    def get_pin(self):
        if self.target == 'micropython':
            from machine import Pin
            return Pin(self.pin_num, Pin.OUT)

        elif self.target == 'adafruit':
            import board
            if self.pin_num == 18:
                return board.D18
            elif self.pin_num == 23:
                return board.D23
            elif self.pin_num == 24:
                return board.D24
            elif self.pin_num == 24:
                return board.D24
            elif self.pin_num == 25:
                return board.D25
            elif self.pin_num == 12:
                return board.D12
            elif self.pin_num == 16:
                return board.D16
            elif self.pin_num == 4:
                return board.D4
            elif self.pin_num == 17:
                return board.D17
            elif self.pin_num == 27:
                return board.D27
            elif self.pin_num == 22:
                return board.D22
            elif self.pin_num == 5:
                return board.D5
            elif self.pin_num == 6:
                return board.D6
            elif self.pin_num == 13:
                return board.D13
            elif self.pin_num == 26:
                return board.D26

    def get_sections(self):
        if self.debug:
            print('NeoPixel().get_sections()')

        sections_length = 15
        sections = []
        counter = 0
        while counter < self.addressable_strip_length:
            sections.append(
                [x for x in range(counter, counter+sections_length)])
            counter += sections_length
        return sections

    def get_led_selectors(self, sections='all'):
        if self.debug:
            print('NeoPixel().get_led_selectors(sections={})'.format(sections))

        if type(sections) == str:
            if sections == 'all':
                return range(self.addressable_strip_length)
            elif sections == 'random':
                return self.sections[randint(0, len(self.sections)-1)]
        else:
            selected_leds = []
            for entry in sections:
                selected_leds += self.sections[entry]

            return selected_leds

    def write(self, s_after_wait=1.0/36.0):
        if self.debug:
            print('NeoPixel().write(s_after_wait={})'.format(s_after_wait))

        if self.test:
            from colr import color
            print(
                ''.join(color('  ', back=(x[0], x[1], x[2])) for x in self.leds), end='\r' if self.overwrite_line and not self.debug else '\n')
        else:
            if self.target == 'micropython':
                self.leds.write()
            elif self.target == 'adafruit':
                self.leds.show()
        time.sleep(s_after_wait)

    def insert_led(self, position=0, rgb=[0, 0, 0]):
        if self.debug:
            print('NeoPixel().insert_led(position={},rgb={})'.format(position, rgb))
        # save state of all leds as list, insert LED at position, then write LEDs
        leds = [[x[0], x[1], x[2]] for x in self.leds]
        leds.insert(position, rgb)
        leds = leds[:-1]
        for i in range(len(leds)):
            self.leds[i] = leds[i]

    def append_led(self, rgb=[0, 0, 0]):
        if self.debug:
            print('NeoPixel().append_led(rgb={})'.format(rgb))
        # save state of all leds as list, append LED at the end, then write LEDs
        leds = [[x[0], x[1], x[2]] for x in self.leds]
        leds.append(rgb)
        leds = leds[1:]
        for i in range(len(leds)):
            self.leds[i] = leds[i]

    def get_led(self, i, start=None):
        if self.debug:
            print('NeoPixel().get_led(i={},start={}'.format(i, start))
        i = i+self.start_led
        if i < 0:
            i += self.addressable_strip_length
        if start and start == 'end':
            i += (self.strip_length-self.addressable_strip_length)

        return i

    def off(self):
        if self.debug:
            print('NeoPixel().off()')
        for i in range(self.strip_length):
            self.leds[i] = (0, 0, 0)
        self.write()

    def on(self, num=None):
        if self.debug:
            print('NeoPixel().on(num={})'.format(num))
        if type(num) == int:
            num = self.get_led(num)
            self.leds[num] = (255, 255, 255)
        else:
            for i in range(self.strip_length):
                self.leds[i] = (255, 255, 255)
        self.write()

    def test_animations(self):
        # run all the animations for testing
        print('Start testing animations...')
        while True:
            self.rainbow_animation(loop_limit=2)

            self.beats(loop_limit=3)
            self.beats(loop_limit=3, start='end')
            self.beats(loop_limit=3, start='start + end')
            self.beats(loop_limit=3, start='center', brightness=0.5)

            self.moving_dot(loop_limit=3)
            self.moving_dot(loop_limit=3, start='end', brightness=0.5)

            self.light_up(loop_limit=3)
            self.light_up(loop_limit=3, sections='random')
            self.light_up(loop_limit=3, sections=[0])

            self.transition(loop_limit=3)
            self.transition(loop_limit=3, sections=[0])

    def color(self,
              rgb_color=None,
              customization_json={}
              ):
        r = customization_json['rgb_color'][0] if customization_json and 'rgb_color' in customization_json else rgb_color[0]
        g = customization_json['rgb_color'][1] if customization_json and 'rgb_color' in customization_json else rgb_color[1]
        b = customization_json['rgb_color'][2] if customization_json and 'rgb_color' in customization_json else rgb_color[2]

        for i in range(self.strip_length):
            i = self.get_led(i)
            self.leds[i] = (r, g, b)
        self.write()

    def rainbow_animation(self,
                          loop_limit=None,
                          brightness=1,
                          duration_ms=1000,
                          pause_ms=None,
                          customization_json={}
                          ):

        RainbowAnimation(
            led_strip=self,
            loop_limit=customization_json['loop_limit'] if customization_json and 'loop_limit' in customization_json else loop_limit,
            brightness=customization_json['brightness'] if customization_json and 'brightness' in customization_json else brightness,
            duration_ms=customization_json['duration_ms'] if customization_json and 'duration_ms' in customization_json else duration_ms,
            pause_ms=customization_json['pause_ms'] if customization_json and 'pause_ms' in customization_json else pause_ms
        ).glow()

    def beats(self,
              rgb_colors=None,
              brightness=1,
              brightness_fixed=False,
              max_height=1,
              loop_limit=None,
              duration_ms=200,
              pause_ms=300,
              start='start',
              num_random_colors=5,
              customization_json={}
              ):

        BeatsUpAndDown(
            led_strip=self,
            rgb_colors=customization_json['rgb_colors'] if customization_json and 'rgb_colors' in customization_json else rgb_colors,
            brightness=customization_json['brightness'] if customization_json and 'brightness' in customization_json else brightness,
            brightness_fixed=customization_json['brightness_fixed'] if customization_json and 'brightness_fixed' in customization_json else brightness_fixed,
            max_height=customization_json['max_height'] if customization_json and 'max_height' in customization_json else max_height,
            loop_limit=customization_json['loop_limit'] if customization_json and 'loop_limit' in customization_json else loop_limit,
            duration_ms=customization_json['duration_ms'] if customization_json and 'duration_ms' in customization_json else duration_ms,
            pause_ms=customization_json['pause_ms'] if customization_json and 'pause_ms' in customization_json else pause_ms,
            start=customization_json['start'] if customization_json and 'start' in customization_json else start,
            num_random_colors=customization_json['num_random_colors'] if customization_json and 'num_random_colors' in customization_json else num_random_colors
        ).glow()

    def moving_dot(self,
                   rgb_colors=None,
                   brightness=1,
                   loop_limit=None,
                   duration_ms=200,
                   pause_a_ms=0,
                   pause_b_ms=300,
                   start='start',
                   num_random_colors=5,
                   customization_json={}
                   ):

        MovingDot(
            led_strip=self,
            rgb_colors=customization_json['rgb_colors'] if customization_json and 'rgb_colors' in customization_json else rgb_colors,
            brightness=customization_json['brightness'] if customization_json and 'brightness' in customization_json else brightness,
            loop_limit=customization_json['loop_limit'] if customization_json and 'loop_limit' in customization_json else loop_limit,
            duration_ms=customization_json['duration_ms'] if customization_json and 'duration_ms' in customization_json else duration_ms,
            pause_a_ms=customization_json['pause_a_ms'] if customization_json and 'pause_a_ms' in customization_json else pause_a_ms,
            pause_b_ms=customization_json['pause_b_ms'] if customization_json and 'pause_b_ms' in customization_json else pause_b_ms,
            start=customization_json['start'] if customization_json and 'start' in customization_json else start,
            num_random_colors=customization_json['num_random_colors'] if customization_json and 'num_random_colors' in customization_json else num_random_colors
        ).glow()

    def light_up(self,
                 rgb_colors=None,
                 brightness=1,
                 loop_limit=None,
                 duration_ms=200,
                 pause_ms=200,
                 num_random_colors=5,
                 sections='all',
                 customization_json={}
                 ):

        LightUp(
            led_strip=self,
            rgb_colors=customization_json['rgb_colors'] if customization_json and 'rgb_colors' in customization_json else rgb_colors,
            brightness=customization_json['brightness'] if customization_json and 'brightness' in customization_json else brightness,
            loop_limit=customization_json['loop_limit'] if customization_json and 'loop_limit' in customization_json else loop_limit,
            duration_ms=customization_json['duration_ms'] if customization_json and 'duration_ms' in customization_json else duration_ms,
            pause_ms=customization_json['pause_ms'] if customization_json and 'pause_ms' in customization_json else pause_ms,
            num_random_colors=customization_json['num_random_colors'] if customization_json and 'num_random_colors' in customization_json else num_random_colors,
            sections=customization_json['sections'] if customization_json and 'sections' in customization_json else sections
        ).glow()

    def transition(self,
                   rgb_colors=None,
                   brightness=1,
                   loop_limit=None,
                   duration_ms=200,
                   pause_ms=200,
                   num_random_colors=5,
                   sections='all',
                   customization_json={}
                   ):

        Transition(
            led_strip=self,
            rgb_colors=customization_json['rgb_colors'] if customization_json and 'rgb_colors' in customization_json else rgb_colors,
            brightness=customization_json['brightness'] if customization_json and 'brightness' in customization_json else brightness,
            loop_limit=customization_json['loop_limit'] if customization_json and 'loop_limit' in customization_json else loop_limit,
            duration_ms=customization_json['duration_ms'] if customization_json and 'duration_ms' in customization_json else duration_ms,
            pause_ms=customization_json['pause_ms'] if customization_json and 'pause_ms' in customization_json else pause_ms,
            num_random_colors=customization_json['num_random_colors'] if customization_json and 'num_random_colors' in customization_json else num_random_colors,
            sections=customization_json['sections'] if customization_json and 'sections' in customization_json else sections
        ).glow()


if __name__ == "__main__":
    test = True
    target = None
    n = 60
    animation = 'rainbow_animation'
    try:
        opts, args = getopt.getopt(sys.argv[1:], "ht:d:n:a:", [
                                   "test=", "target=", "n=", "animation="])
    except getopt.GetoptError:
        print('neopixel_plus.py -t <test> -d <target> -n <n> -a <animation>')
        sys.exit(2)
    for opt, arg in opts:
        if opt in ("-t", "--test"):
            test = arg
        elif opt in ("-d", "--target"):
            target = arg
        elif opt in ("-n", "--n"):
            n = int(arg)
        elif opt in ("-a", "--animation"):
            animation = arg

    if target:
        test = False

    # get parameters and function call from shell commands
    getattr(NeoPixel(test=test, target=target, n=n), animation)()



import math
import time

# TODO add Color() and simplify code (create rainbow once, then move it)


class RainbowAnimation:
    def __init__(self,
                 led_strip,
                 brightness=1,
                 loop_limit=None,
                 duration_ms=1000,
                 pause_ms=None):
        self.led_strip = led_strip
        self.brightness_max = brightness
        self.loop_limit = loop_limit
        self.duration_ms = duration_ms
        self.pause_ms = pause_ms

        self.time_passed_ms = 0

    def set_time_passed_ms(self):
        if self.led_strip.debug:
            print('RainbowAnimation().set_time_passed_ms()')

        # if duration 1000 ms = 17 loops * 60ms
        # if duration 500 ms = 8.5 loops 120ms
        # if duration 250 ms = 4.25 loops 240ms
        # if duration 100 ms = 1.7 loops 600ms
        full_duration = 1000
        default_rate = 60

        added_ms = (full_duration/self.duration_ms)*default_rate

        self.time_passed_ms += added_ms

    def set_brightness(self, counter, max_counter):
        if self.led_strip.debug:
            print('RainbowAnimation().set_brightness(counter={},max_counter={})'.format(
                counter, max_counter))

        # if counter == 1 => brightness 0.3
        # if counter == 2 => brightness 0.6
        # if counter == max_counter-1 => brightness 0.3
        # if counter == max_counter-2 => brightness 0.6
        if self.duration_ms and self.pause_ms and counter == 0:
            self.brightness = 0.3*self.brightness_max
        elif self.duration_ms and self.pause_ms and counter == 1:
            self.brightness = 0.6*self.brightness_max
        elif self.duration_ms and self.pause_ms and counter == (max_counter-2):
            self.brightness = 0.6*self.brightness_max
        elif self.duration_ms and self.pause_ms and counter == (max_counter-1):
            self.brightness = 0.3*self.brightness_max
        else:
            self.brightness = 1*self.brightness_max

    def get_max_counter(self):
        if self.led_strip.debug:
            print('RainbowAnimation().get_max_counter()')

        # if duration 1000 ms = 17 loops * 60ms
        # if duration 500 ms = 8.5 loops 120ms
        # if duration 250 ms = 4.25 loops 240ms
        # if duration 100 ms = 1.7 loops 600ms
        full_duration = 1000
        loops = 17

        return round((self.duration_ms/full_duration)*loops)

    def glow(self):
        if self.led_strip.debug:
            print('RainbowAnimation().glow()')

        print('Rainbow:')
        try:
            # if duration, need to adapt time_passed to make one full color loop (and then pause if pause set)
            # turn LEDs rainbow
            counter = 0
            loops = 0
            max_counter = self.get_max_counter()
            while True:
                self.set_brightness(counter, max_counter)

                # turn LEDs black (off) for duration of pause
                if counter == max_counter:
                    if self.duration_ms and self.pause_ms:
                        self.led_strip.off()
                        time.sleep((self.pause_ms-10)/1000)

                    counter = 0
                    loops += 1

                    if self.loop_limit and self.loop_limit == loops:
                        print()
                        break

                else:

                    self.set_time_passed_ms()
                    for i in range(self.led_strip.strip_length):
                        i = self.led_strip.get_led(i)
                        color = self.rainbow_color(self.time_passed_ms, i,
                                                   self.brightness)
                        self.led_strip.leds[i] = color

                    self.led_strip.write()

                    counter += 1
        except KeyboardInterrupt:
            import sys
            print()
            sys.exit(0)

    def rainbow_color(self, t, i, brightness):
        if self.led_strip.debug:
            print('RainbowAnimation().rainbow_color(t={},i={},brightness={})'.format(
                t, i, brightness))

        t = t/1000
        a = (0.5, 0.5, 0.5)
        b = (0.5, 0.5, 0.5)
        c = (1.0, 1.0, 1.0)
        d = (0.00, 0.33, 0.67)

        k = t + 0.05 * i

        r = a[0] + b[0] * math.cos(6.28318 * (c[0] * k + d[0]))
        g = a[1] + b[1] * math.cos(6.28318 * (c[1] * k + d[1]))
        b = a[2] + b[2] * math.cos(6.28318 * (c[2] * k + d[2]))

        r = int(255.0 * r * brightness)
        g = int(255.0 * g * brightness)
        b = int(255.0 * b * brightness)

        return (r if r < 255 else 255, g if g < 255 else 255, b if b < 255 else 255)



import time
from os import name

try:
    from neopixel_plus.helper import Color
except ImportError:
    from helper import Color


class Transition:
    def __init__(self,
                 led_strip,
                 rgb_colors=None,
                 brightness=1,
                 loop_limit=None,
                 duration_ms=200,
                 pause_ms=200,
                 num_random_colors=5,
                 sections='all'):
        self.led_strip = led_strip
        self.loop_limit = loop_limit
        self.loops = 0
        self.duration_ms = duration_ms
        self.pause_ms = pause_ms
        self.sections = sections
        self.colors = Color(
            rgb_colors=rgb_colors,
            brightness=brightness,
            num_random_colors=num_random_colors
        )

        self.transition_steps = 20
        self.write_wait_time = (self.duration_ms/self.transition_steps)/1000

        self.selected_leds = self.led_strip.get_led_selectors(
            self.sections)

    def glow(self):
        if self.led_strip.debug:
            print('Transition().glow()')

        print('Transition:')
        try:
            # make sure leds are off
            self.led_strip.off()

            while True:
                # add or substract difference between rgb values and update color - to make transition

                steps_counter = 0
                target_color = self.colors.selected
                difference = [
                    target_color[0] -
                    self.led_strip.leds[self.selected_leds[0]][0],
                    target_color[1] -
                    self.led_strip.leds[self.selected_leds[0]][1],
                    target_color[2] -
                    self.led_strip.leds[self.selected_leds[0]][2],
                ]

                difference_per_step = [
                    round(difference[0]/self.transition_steps),
                    round(difference[1]/self.transition_steps),
                    round(difference[2]/self.transition_steps),
                ]

                while steps_counter != self.transition_steps:

                    self.colors.selected = [
                        self.led_strip.leds[self.selected_leds[0]
                                            ][0]+difference_per_step[0],
                        self.led_strip.leds[self.selected_leds[0]
                                            ][1]+difference_per_step[1],
                        self.led_strip.leds[self.selected_leds[0]
                                            ][2]+difference_per_step[2],
                    ]

                    # make to not select invalid RGB values
                    new_list = []
                    for number in self.colors.selected:
                        if number < 0:
                            new_list.append(0)
                        elif number > 255:
                            new_list.append(255)
                        else:
                            new_list.append(number)
                    self.colors.selected = new_list

                    for i in self.selected_leds:
                        self.led_strip.leds[i] = self.colors.selected
                    self.led_strip.write(s_after_wait=self.write_wait_time)

                    steps_counter += 1

                # change to next color
                self.colors.next()

                if self.pause_ms:
                    time.sleep(self.pause_ms/1000)

                self.loops += 1
                if self.loop_limit and self.loop_limit == self.loops:
                    print()
                    break

        except KeyboardInterrupt:
            import sys
            print()
            sys.exit(0)



import time

try:
    from neopixel_plus.helper import Color
except ImportError:
    from helper import Color


class MovingDot:
    def __init__(self,
                 led_strip,
                 loop_limit=None,
                 duration_ms=200,
                 pause_a_ms=0,
                 pause_b_ms=300,
                 start='start',
                 rgb_colors=None,
                 brightness=1,
                 num_random_colors=5):
        self.led_strip = led_strip
        self.loop_limit = loop_limit
        self.loops = 0
        self.duration_ms = duration_ms
        self.pause_a_ms = pause_a_ms
        self.pause_b_ms = pause_b_ms
        self.start = start
        self.selector = {
            "start": [-1, -2, -3, -4, -5],
            "end": [0, 1, 2, 3, 4]
        }

        self.colors = Color(
            rgb_colors=rgb_colors,
            brightness=brightness,
            num_random_colors=num_random_colors
        )

        self.write_wait_time = (
            self.duration_ms/2/self.led_strip.strip_length)/1000

    def create_dot(self):
        if self.led_strip.debug:
            print('MovingDot().create_dot()')

        self.dot = [self.colors.black]*5

        counter = 0
        for selected in self.selector[self.start]:
            self.colors.brightness = (
                1-(counter*0.225)) * self.colors.brightness_max
            self.colors.correct()
            self.dot[selected] = self.colors.selected
            counter += 1

    def change_direction(self):
        if self.led_strip.debug:
            print('MovingDot().change_direction()')

        if self.start == 'start':
            self.start = 'end'
        else:
            self.start = 'start'

    def move_dot(self):
        if self.led_strip.debug:
            print('MovingDot().move_dot()')

        # move dot into view
        for selected in self.selector[self.start]:
            if self.start == 'start':
                self.led_strip.insert_led(0, self.dot[selected])
            else:
                self.led_strip.append_led(self.dot[selected])
            self.led_strip.write(s_after_wait=self.write_wait_time)

        # add black led to front and remove last led, to move dot
        while True:
            if self.start == 'start':
                self.led_strip.insert_led()
            else:
                self.led_strip.append_led()
            self.led_strip.write(s_after_wait=self.write_wait_time)

            # if all leds black, exit loop
            for led in self.led_strip.leds:
                if led[0] != 0 or led[1] != 0 or led[2] != 0:
                    break
            else:
                break

    def glow(self):
        if self.led_strip.debug:
            print('MovingDot().glow()')

        print('Moving dot:')
        try:
            # make sure leds are off
            self.led_strip.off()

            while True:
                # make sure duration is correct
                # create dot with tail
                self.create_dot()

                # move dot with tail and write
                self.move_dot()

                # once dot disappeared at the end: pause_a
                time.sleep(self.pause_a_ms/1000)

                # create new dot with tail and move in opposit start
                self.change_direction()
                self.create_dot()
                self.move_dot()
                self.change_direction()

                # once dot disappeared at the end: pause_b
                time.sleep(self.pause_b_ms/1000)

                # change to next color
                self.colors.next()

                self.loops += 1
                if self.loop_limit and self.loop_limit == self.loops:
                    print()
                    break

        except KeyboardInterrupt:
            import sys
            print()
            sys.exit(0)
